/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0 and the Server Side Public License, v 1; you may not use this file except
 * in compliance with, at your election, the Elastic License 2.0 or the Server
 * Side Public License, v 1.
 */

import os from 'os';

import normalizePath from 'normalize-path';

const HOME_DIR = normalizePath(os.homedir());

export const SCRIPT_SOURCE = `#!/usr/bin/env bash
#
# ** THIS IS AN AUTO-GENERATED FILE **
# ** PLEASE DO NOT CHANGE IT MANUALLY **
#
# GENERATED BY \`node scripts/register_git_hook\`
# IF YOU WANNA CHANGE SOMETHING IN THIS SCRIPT
# PLEASE RE-RUN 'yarn kbn bootstrap' or 'node scripts/register_git_hook'

# pre-commit script takes zero arguments: https://git-scm.com/docs/githooks#_pre_commit

set -euo pipefail

# Make it possible to terminate pre commit hook
# using ctrl-c so nothing else would happen or be
# sent to the output.
#
# The correct exit code on that situation
# according the linux documentation project is 130
# https://www.tldp.org/LDP/abs/html/exitcodes.html
trap "exit 130" INT

has_node() {
  command -v node >/dev/null 2>&1
}

has_nvm() {
  command -v nvm >/dev/null 2>&1
}

try_load_node_from_nvm_paths () {
  # If nvm is not loaded, load it
  has_node || {
    NVM_SH="${HOME_DIR}/.nvm/nvm.sh"

    if [ "${process.platform}" == "darwin" ] && [ -s "$(brew --prefix nvm)/nvm.sh" ]; then
      NVM_SH="$(brew --prefix nvm)/nvm.sh"
    fi

    export NVM_DIR="${HOME_DIR}/.nvm"

    [ -s "$NVM_SH" ] && \. "$NVM_SH"

    # If nvm has been loaded correctly, use project .nvmrc
    has_nvm && nvm use
  }
}

extend_user_path() {
  if [ "${process.platform}" == "win32" ]; then
    export PATH="$PATH:/c/Program Files/nodejs"
  else
    export PATH="$PATH:/usr/local/bin:/usr/local"
    try_load_node_from_nvm_paths
  fi
}

# Extend path with common path locations for node
# in order to make the hook working on git GUI apps
extend_user_path

# Check if we have node js bin in path
has_node || {
  echo "Can't found node bin in the PATH. Please update the PATH to proceed."
  echo "If your PATH already has the node bin, maybe you are using some git GUI app."
  echo "Can't found node bin in the PATH. Please update the PATH to proceed."
  echo "If your PATH already has the node bin, maybe you are using some git GUI app not launched from the shell."
  echo "In order to proceed, you need to config the PATH used by the application that are launching your git GUI app."
  echo "If you are running macOS, you can do that using:"
  echo "'sudo launchctl config user path /usr/local/bin:/usr/bin:/bin:/usr/sbin:/sbin'"

  exit 1
}

execute_precommit_hook() {
  node scripts/precommit_hook || return 1

  PRECOMMIT_FILE="./.git/hooks/pre-commit.local"
  if [ -x "\${PRECOMMIT_FILE}" ]; then
    echo "Executing local precommit hook found in \${PRECOMMIT_FILE}"
    "$PRECOMMIT_FILE" || return 1
  fi
}

execute_precommit_hook || {
  echo "Pre-commit hook failed (add --no-verify to bypass)";
  echo '  For eslint failures you can try running \`node scripts/precommit_hook --fix\`';
  exit 1;
}

exit 0
`;
